@@ -88,7 +88,7 @@ formatting = {
     'speech': {
         'prefix': {
             'focused': '[]',
-            'unfocused': 'newAncestors + newRowHeader + newColumnHeader + newRadioButtonGroup',
+            'unfocused': 'oldAncestors + newAncestors + newRowHeader + newColumnHeader + newRadioButtonGroup',
             'basicWhereAmI': 'toolbar',
             'detailedWhereAmI' : '[]'
             },
@@ -100,7 +100,7 @@ formatting = {
             },
         'default': {
             'focused': '[]',
-            'unfocused': 'labelAndName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator',
+            'unfocused': 'labelAndName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator + childWidget',
             'basicWhereAmI': 'labelAndName + roleName',
             'detailedWhereAmI' : '[]'
             },
@@ -154,7 +154,7 @@ formatting = {
             'detailedWhereAmI': 'label + placeholderText + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
         pyatspi.ROLE_FRAME: {
-            'focused': '[]',
+            'focused': 'labelAndName + roleName',
             'unfocused': 'labelAndName + allTextSelection + roleName + unfocusedDialogCount + availability'
             },
         pyatspi.ROLE_HEADING: {
@@ -186,7 +186,7 @@ formatting = {
             },
         pyatspi.ROLE_LIST_ITEM: {
             'focused': 'expandableState + availability',
-            'unfocused': 'labelAndName + allTextSelection + expandableState + availability + positionInList',
+            'unfocused': 'labelAndName + allTextSelection + expandableState + availability + positionInList + childWidget',
             'basicWhereAmI': 'label + roleName + name + positionInList + expandableState + (nodeLevel or nestingLevel)'
             },
         pyatspi.ROLE_MENU: {
@@ -195,8 +195,8 @@ formatting = {
             'basicWhereAmI': '(ancestors or parentRoleName) + labelAndName + roleName +  positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_MENU_ITEM: {
-            'focused': '[]',
-            'unfocused': 'labelAndName + menuItemCheckedState + availability + ' + MNEMONIC + ' + accelerator + positionInList',
+            'focused': 'expandableState',
+            'unfocused': 'labelAndName + menuItemCheckedState + expandableState + availability + ' + MNEMONIC + ' + accelerator + positionInList',
             'basicWhereAmI': 'ancestors + labelAndName + accelerator + positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_NOTIFICATION: {
@@ -224,6 +224,7 @@ formatting = {
             'unfocused': 'labelAndName + percentage'
             },
         pyatspi.ROLE_PUSH_BUTTON: {
+            'focused': 'expandableState',
             'unfocused': 'labelAndName + expandableState + roleName + availability + ' + MNEMONIC + ' + accelerator',
             'basicWhereAmI': 'labelAndName + expandableState + roleName + ' + MNEMONIC + ' + accelerator'
             },
@@ -402,6 +403,8 @@ formatting = {
             'unfocused': BRAILLE_TEXT
             },
         pyatspi.ROLE_FRAME: {
+            'focused':   '[Component(obj,\
+                                     asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]',
             'unfocused': '[Component(obj,\
                                      asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]'
             },
@@ -437,10 +440,12 @@ formatting = {
         pyatspi.ROLE_LIST_ITEM: {
             'focused':   '[Component(obj,\
                                      asString(label + displayedText + expandableState + roleName + availability) + asString(accelerator))]\
-                          + (nestingLevel and [Region(" " + asString(nestingLevel))])',
+                          + (nestingLevel and [Region(" " + asString(nestingLevel))])\
+                          + (childWidget and ([Region(" ")] + childWidget))',
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + expandableState))]\
-                          + (nestingLevel and [Region(" " + asString(nestingLevel))])',
+                          + (nestingLevel and [Region(" " + asString(nestingLevel))])\
+                          + (childWidget and ([Region(" ")] + childWidget))',
             },
         pyatspi.ROLE_MENU: {
             'focused':   '[Component(obj,\
@@ -451,7 +456,7 @@ formatting = {
         #pyatspi.ROLE_MENU_BAR: 'default'
         pyatspi.ROLE_MENU_ITEM: {
             'unfocused': '[Component(obj,\
-                                     asString(label + displayedText + availability) + asString(accelerator),\
+                                     asString(label + displayedText + expandableState + availability) + asString(accelerator),\
                                      indicator=asString(menuItemCheckedState))]'
             },
         #pyatspi.ROLE_OPTION_PANE: 'default'
@@ -464,7 +469,8 @@ formatting = {
         #pyatspi.ROLE_PAGE_TAB_LIST: 'default'
         pyatspi.ROLE_PANEL: {
             'unfocused': '[Component(obj,\
-                                     asString((label or displayedText) + roleName))]'
+                                     asString((label or displayedText) + roleName))]\
+                          + (childWidget and ([Region(" ")] + childWidget))'
             },
         pyatspi.ROLE_PARAGRAPH: {
             'unfocused': BRAILLE_TEXT
@@ -581,13 +587,13 @@ if settings.useExperimentalSpeechProsody:
     formatting['speech'][pyatspi.ROLE_LIST]['unfocused'] = \
         'labelOrName + pause + focusedItem + pause + multiselectableState + numberOfChildren + pause'
     formatting['speech'][pyatspi.ROLE_LIST_ITEM]['unfocused'] = \
-        'labelAndName + allTextSelection + pause + expandableState + pause + availability + positionInList'
+        'labelAndName + allTextSelection + pause + expandableState + pause + availability + positionInList + pause + childWidget'
     formatting['speech'][pyatspi.ROLE_LIST_ITEM]['basicWhereAmI'] = \
         'label + roleName + pause + name + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
     formatting['speech'][pyatspi.ROLE_MENU]['unfocused'] = 'labelAndName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList'
     formatting['speech'][pyatspi.ROLE_MENU]['basicWhereAmI'] = \
         '(ancestors or parentRoleName) + pause + labelAndName + roleName + pause +  positionInList + ' + MNEMONIC
-    formatting['speech'][pyatspi.ROLE_MENU_ITEM]['unfocused'] = 'labelAndName + menuItemCheckedState + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList'
+    formatting['speech'][pyatspi.ROLE_MENU_ITEM]['unfocused'] = 'labelAndName + menuItemCheckedState + expandableState + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList'
     formatting['speech'][pyatspi.ROLE_MENU_ITEM]['basicWhereAmI'] = \
         'ancestors + pause + labelAndName + pause + accelerator + pause + positionInList + ' + MNEMONIC
     formatting['speech'][pyatspi.ROLE_PAGE_TAB]['focused'] = 'labelAndName + roleName + pause + positionInList + ' + MNEMONIC + ' + accelerator'
