@@ -1,5 +1,4 @@
 __all__ = ['Banshee',
-           'ddu',
            'Eclipse',
            'ekiga',
            'empathy',
@@ -9,13 +8,13 @@ __all__ = ['Banshee',
            'gcalctool',
            'gdmlogin',
            'gedit',
+           'gnome-documents',
            'gnome-keyring-ask',
            'gnome-mud',
            'gnome-panel',
            'gnome-screensaver-dialog',
            'gnome-search-tool',
-           'gnome_segv2',
-           'gnome-system-monitor',
+           'gnome-shell',
            'gnome-terminal',
            'gnome-window-properties',
            'gtk-window-decorator',
@@ -23,10 +22,8 @@ __all__ = ['Banshee',
            'liferea',
            'metacity',
            'Mozilla',
-           'nautilus',
            'notification-daemon',
            'notify-osd',
-           'packagemanager',
            'pidgin',
            'planner',
            'rhythmbox',
