@@ -28,16 +28,14 @@ __license__   = "LGPL"
 import pyatspi
 
 import orca.debug as debug
-import orca.scripts.default as default
-import orca.orca as orca
 import orca.orca_state as orca_state
+import orca.scripts.toolkits.gtk as gtk
 import orca.settings as settings
 import orca.speech as speech
-import orca.speechserver as speechserver
 
 from orca.orca_i18n import _
 
-class Script(default.Script):
+class Script(gtk.Script):
 
     def __init__(self, app):
         """Creates a new script for the given application.
@@ -46,7 +44,7 @@ class Script(default.Script):
         - app: the application to create a script for.
         """
 
-        default.Script.__init__(self, app)
+        gtk.Script.__init__(self, app)
 
         # Set the debug level for all the methods in this script.
         #
@@ -65,135 +63,6 @@ class Script(default.Script):
         self.lastBadWord = ''
         self.lastEventType = ''
 
-    def getListeners(self):
-        """Sets up the AT-SPI event listeners for this script.
-        """
-        listeners = default.Script.getListeners(self)
-
-        listeners["object:state-changed:focused"]           = \
-            self.onStateChanged
-
-        return listeners
-
-    def __sayAllProgressCallback(self, context, progressType):
-        """Provide feedback during the sayAll operation.
-        """
-
-        if progressType == speechserver.SayAllContext.PROGRESS:
-            #print "PROGRESS", context.utterance, context.currentOffset
-            pass
-        elif progressType == speechserver.SayAllContext.INTERRUPTED:
-            #print "INTERRUPTED", context.utterance, context.currentOffset
-            offset = context.currentOffset
-            for i in range(0, len(context.obj)):
-                obj = context.obj[i]
-                text = obj.queryText()
-                charCount = text.characterCount
-                if offset > charCount:
-                    offset -= charCount
-                else:
-                    text.setCaretOffset(offset)
-                    break
-        elif progressType == speechserver.SayAllContext.COMPLETED:
-            #print "COMPLETED", context.utterance, context.currentOffset
-            obj = context.obj[len(context.obj)-1]
-            obj.queryText().setCaretOffset(context.currentOffset)
-            orca.setLocusOfFocus(None, obj, notifyScript=False)
-
-    def textLines(self, obj):
-        """Creates a generator that can be used to iterate over each line
-        of a text object, starting at the caret offset.
-
-        Arguments:
-        - obj: an Accessible that has a text specialization
-
-        Returns an iterator that produces elements of the form:
-        [SayAllContext, acss], where SayAllContext has the text to be
-        spoken and acss is an ACSS instance for speaking the text.
-        """
-
-        try:
-            text = obj.queryText()
-        except:
-            return
-
-        length = text.characterCount
-        offset = text.caretOffset
-        string = ""
-        textObjs = []
-        textObjs.append(obj)
-        startOffset = text.caretOffset
-
-        # Determine the correct "say all by" mode to use.
-        #        
-        if settings.sayAllStyle == settings.SAYALL_STYLE_SENTENCE:
-            mode = pyatspi.TEXT_BOUNDARY_SENTENCE_END
-        elif settings.sayAllStyle == settings.SAYALL_STYLE_LINE:
-            mode = pyatspi.TEXT_BOUNDARY_LINE_START
-        else:
-            mode = pyatspi.TEXT_BOUNDARY_LINE_START
-
-        # Get the next line of text to read
-        #
-        done = False
-        while not done:
-            while offset < length:
-                [mystr, start, end] = text.getTextAtOffset(offset, mode)
-                endOffset = end
-
-                if len(mystr) != 0:
-                    string += mystr
-
-                if mode == pyatspi.TEXT_BOUNDARY_LINE_START or \
-                   len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
-                    string = self.utilities.adjustForRepeats(string)
-                    if string.isupper():
-                        voice = settings.voices[settings.UPPERCASE_VOICE]
-                    else:
-                        voice = settings.voices[settings.DEFAULT_VOICE]
-
-                    yield [speechserver.SayAllContext(textObjs, string,
-                                                      startOffset, endOffset),
-                           voice]
-                    string = ""
-                    startOffset = endOffset
-
-                if len(mystr) == 0:
-                    break
-                else:
-                    offset = end
-
-            moreLines = False
-            relations = obj.getRelationSet()
-            for relation in relations:
-                if relation.getRelationType()  \
-                       == pyatspi.RELATION_FLOWS_TO:
-                    obj = relation.getTarget(0)
-                    try:
-                        text = obj.queryText()
-                    except:
-                        return
-                    textObjs.append(obj)
-                    length = text.characterCount
-                    offset = 0
-                    moreLines = True
-                    break
-            if not moreLines:
-                done = True
-
-        # If there is anything left unspoken, speak it now.
-        #
-        if len(string) != 0:
-            string = self.utilities.adjustForRepeats(string)
-            if string.isupper():
-                voice = settings.voices[settings.UPPERCASE_VOICE]
-            else:
-                voice = settings.voices[settings.DEFAULT_VOICE]
-
-            yield [speechserver.SayAllContext(textObjs, string,
-                                              startOffset, endOffset),
-                   voice]
-
     def readMisspeltWord(self, event, panel):
         """Speak/braille the current misspelt word plus its context.
            The spell check dialog contains a "paragraph" which shows the
@@ -397,7 +266,7 @@ class Script(default.Script):
         # For everything else, pass the focus event onto the parent class
         # to be handled in the default way.
 
-        default.Script.locusOfFocusChanged(self, event,
+        gtk.Script.locusOfFocusChanged(self, event,
                                            oldLocusOfFocus, newLocusOfFocus)
 
         # If we are doing a Print Preview and we are focused on the
@@ -413,8 +282,6 @@ class Script(default.Script):
                      pyatspi.ROLE_PAGE_TAB,
                      pyatspi.ROLE_PAGE_TAB_LIST]
         if self.utilities.hasMatchingHierarchy(event.source, rolesList):
-            debug.println(self.debugLevel,
-                          "gedit.onStateChanged - print preview - page #.")
             parent = event.source.parent
             label1 = self.utilities.displayedText(parent[1])
             label2 = self.utilities.displayedText(parent[2])
@@ -497,28 +364,7 @@ class Script(default.Script):
             speech.speak(event.source.name)
 
         # Pass the event onto the parent class to be handled in the default way.
-        default.Script.onNameChanged(self, event)
-
-    def onStateChanged(self, event):
-        """Called whenever an object's state changes.
-
-        Arguments:
-        - event: the Event
-        """
-
-        # Sometimes an object will tell us it is focused this
-        # way versus issuing a focus event.  GEdit's edit area,
-        # for example, will do this: when you use metacity's
-        # window menu to do things like maximize or unmaximize
-        # a window, you will only get a state-changed event
-        # from the text area when it regains focus.
-        # (See bug #350854 for more details).
-        #
-        if event.type.startswith("object:state-changed:focused") \
-           and (event.detail1):
-            orca.setLocusOfFocus(event, event.source)
-
-        default.Script.onStateChanged(self, event)
+        gtk.Script.onNameChanged(self, event)
 
     # This method tries to detect and handle the following cases:
     # 1) find dialog - phrase found.
@@ -567,4 +413,4 @@ class Script(default.Script):
         # For everything else, pass the caret moved event onto the parent
         # class to be handled in the default way.
 
-        default.Script.onCaretMoved(self, event)
+        gtk.Script.onCaretMoved(self, event)
