@@ -70,9 +70,45 @@ class Script(default.Script):
         # https://bugzilla.gnome.org/show_bug.cgi?id=711397
         if role == pyatspi.ROLE_COMBO_BOX:
             orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Unfiled, but a similar case of the above issue with combo boxes.
+        if role == pyatspi.ROLE_PUSH_BUTTON:
+            orca.setLocusOfFocus(event, event.source)
+            return
 
         # Unfiled. But this happens when you are in Gedit, get into a menu
         # and then press Escape. The text widget emits a focus: event, but
         # not a state-changed:focused event.
-        if role == pyatspi.ROLE_TEXT:
+        #
+        # A similar issue can be seen when a text widget starts out having
+        # focus, such as in the old gnome-screensaver dialog.
+        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
             orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Unfiled. When a context menu first appears and an item is already
+        # selected, we get a focus: event for that menu item, but there is
+        # not a state-changed event for that item, nor a selection-changed
+        # event for the menu.
+        if role == pyatspi.ROLE_MENU_ITEM:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Unfiled. When a canvas item gets focus but is not selected, we
+        # are only getting a focus event. This happens in Nautilus.
+        if role == pyatspi.ROLE_CANVAS:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        # Unfiled, but yet another case of only getting a focus: event when
+        # a widget appears in a parent container and is already focused.
+        # An example of this particular case is the list of elements dialogs.
+        if role == pyatspi.ROLE_TABLE:
+            obj = event.source
+            selectedChildren = self.utilities.selectedChildren(obj)
+            if selectedChildren:
+                obj = selectedChildren[0]
+
+            orca.setLocusOfFocus(event, obj)
+            return
