@@ -27,14 +27,13 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import importlib
-import logging
-log = logging.getLogger("speech")
 
 import re
 import time
 
 from . import chnames
 from . import debug
+from . import logger
 from . import orca_state
 from . import settings
 from . import sound
@@ -42,6 +41,9 @@ from . import speech_generator
 
 from .acss import ACSS
 
+_logger = logger.getLogger()
+log = _logger.newLog("speech")
+
 # The speech server to use for all speech operations.
 #
 _speechserver = None
@@ -304,27 +306,23 @@ def stop():
     if _speechserver:
         _speechserver.stop()
 
+def updateCapitalizationStyle(script=None, inputEvent=None):
+    if _speechserver:
+        _speechserver.updateCapitalizationStyle()
 
+    return True
 
 def updatePunctuationLevel(script=None, inputEvent=None):
     """ Punctuation level changed, inform this speechServer. """
 
     if _speechserver:
         _speechserver.updatePunctuationLevel()
-    else:
-        logLine = "SPEECH OUTPUT: 'punctuation level' updated"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
 def increaseSpeechRate(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechRate()
-    else:
-        logLine = "SPEECH OUTPUT: 'faster'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
@@ -341,20 +339,12 @@ def decreaseSpeechRate(script=None, inputEvent=None):
 def increaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechPitch()
-    else:
-        logLine = "SPEECH OUTPUT: 'higher'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
 def decreaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.decreaseSpeechPitch()
-    else:
-        logLine = "SPEECH OUTPUT: 'lower'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
