@@ -257,11 +257,11 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(SYSTEM)
         role = args.get('role', obj.getRole())
 
-        doNotPresent = [pyatspi.ROLE_UNKNOWN]
+        doNotPresent = [pyatspi.ROLE_UNKNOWN, pyatspi.ROLE_FILLER]
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
-            doNotPresent.append(pyatspi.ROLE_FILLER)
+            doNotPresent.append(obj.getRole())
 
         if _settingsManager.getSetting('speechVerbosityLevel') \
                 == settings.VERBOSITY_LEVEL_BRIEF:
@@ -916,7 +916,9 @@ class SpeechGenerator(generator.Generator):
         """
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateCurrentLineText(self, obj, **args)
-        if result:
+        if result and result[0]:
+            if result[0] == "\n":
+                result[0] = messages.BLANK
             result.extend(acss)
         return result
 
@@ -1393,35 +1395,26 @@ class SpeechGenerator(generator.Generator):
         and the position of the current item. This object will be an icon
         panel or a layered pane.
         """
+
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        container = obj
+        if not 'Selection' in pyatspi.listInterfaces(container):
+            container = obj.parent
+            if not 'Selection' in pyatspi.listInterfaces(container):
+                return []
+
         result = []
         acss = self.voice(SYSTEM)
-        # TODO - JD: Is there a better way to do this other than
-        # hard-coding it in?
-        #
-        if args.get('role', obj.getRole()) \
-                in [pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS]:
-            obj = obj.parent
-        childCount = obj.childCount
-        selectedItems = []
-        totalSelectedItems = 0
-        currentItem = 0
-        for child in obj:
-            state = child.getState()
-            if state.contains(pyatspi.STATE_SELECTED):
-                totalSelectedItems += 1
-                selectedItems.append(child)
-            if state.contains(pyatspi.STATE_FOCUSED):
-                currentItem = child.getIndexInParent() + 1
-        countString = messages.selectedItemsCount(totalSelectedItems, childCount)
-        result.append(countString)
+        childCount = container.childCount
+        selectedCount = len(self._script.utilities.selectedChildren(container))
+        result.append(messages.selectedItemsCount(selectedCount, childCount))
         result.extend(acss)
         result.append(self._script.formatting.getString(
                           mode='speech',
                           stringType='iconindex') \
-                      % {"index" : currentItem,
+                      % {"index" : obj.getIndexInParent() + 1,
                          "total" : childCount})
         result.extend(acss)
         return result
@@ -1431,16 +1424,18 @@ class SpeechGenerator(generator.Generator):
         specifications) containing the names of all the selected items.
         This object will be an icon panel or a layered pane.
         """
-        result = []
-        # TODO - JD: Is there a better way to do this other than
-        # hard-coding it in?
-        #
-        if args.get('role', obj.getRole()) == pyatspi.ROLE_ICON:
-            obj = obj.parent
-        for child in obj:
-            if child.getState().contains(pyatspi.STATE_SELECTED):
-                result.extend(self._generateLabelAndName(child))
-        return result
+
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        container = obj
+        if not 'Selection' in pyatspi.listInterfaces(container):
+            container = obj.parent
+            if not 'Selection' in pyatspi.listInterfaces(container):
+                return []
+
+        selectedItems = self._script.utilities.selectedChildren(container)
+        return list(map(self._generateLabelAndName, selectedItems))
 
     def _generateUnfocusedDialogCount(self, obj,  **args):
         """Returns an array of strings (and possibly voice and audio
@@ -1480,50 +1475,34 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
-        acss = self.voice(DEFAULT)
         priorObj = args.get('priorObj', None)
-        requireText = args.get('requireText', True)
         commonAncestor = self._script.utilities.commonAncestor(priorObj, obj)
+        try:
+            role = commonAncestor.getRole()
+        except:
+            pass
+        else:
+            if role == pyatspi.ROLE_COMBO_BOX:
+                return []
+
         if obj != commonAncestor:
             parent = obj.parent
-            if parent \
-                and (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                and (parent.getRole() == pyatspi.ROLE_TABLE_CELL):
-                parent = parent.parent
-            while parent and (parent.parent != parent):
-                if parent == commonAncestor:
+            while parent and not parent in [commonAncestor, parent.parent]:
+                if parent.getRole() == pyatspi.ROLE_APPLICATION:
                     break
                 if not self._script.utilities.isLayoutOnly(parent):
-                    text = self._script.utilities.displayedLabel(parent)
-                    if not text \
-                       and (not requireText \
-                            or (requireText \
-                                and 'Text' in pyatspi.listInterfaces(parent))):
-                        text = self._script.utilities.displayedText(parent)
-                    try:
-                        pRole = parent.getRole()
-                    except:
-                        pRole = None
-                    if not text \
-                       and pRole in [pyatspi.ROLE_MENU, pyatspi.ROLE_PAGE_TAB]:
-                        text = parent.name
-                    if text and len(text.strip()):
-                        roleInfo = self._generateRoleName(parent)
-                        if roleInfo:
-                            roleInfo.reverse()
-                        # Push announcement of cell to the end
-                        #
-                        if pRole not in \
-                               [pyatspi.ROLE_TABLE_CELL, pyatspi.ROLE_FILLER]:
-                            result.extend(roleInfo)
-                        result.extend(acss)
-                        result.append(text)
-                        if pRole == pyatspi.ROLE_TABLE_CELL:
-                            result.extend(roleInfo)
+                    result.append(self.generate(parent))
                 parent = parent.parent
         result.reverse()
         return result
 
+    def _generateOldAncestors(self, obj, **args):
+        """Returns an array of strings (and possibly voice and audio
+        specifications) that represent the text of the ancestors for
+        the object being left."""
+
+        return []
+
     def _generateNewAncestors(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the text of the ancestors for
